@@ -20,6 +20,6 @@ namespace StOpt
 /// \param p_values local values associated to the processor
 /// \param p_bdimToSplit    Dimensions to split for parallelism
 Eigen::ArrayXd  reconstructProc0Mpi(const Eigen::ArrayXd &p_point, const std::shared_ptr< FullGrid> &p_grid, const std::shared_ptr< Eigen::ArrayXXd >    &p_values,
-                                    const Eigen::Array< bool, Eigen::Dynamic, 1>   &p_bdimToSplit);
+                                    const Eigen::Array< bool, Eigen::Dynamic, 1>   &p_bdimToSplit, const boost::mpi::communicator &p_world);
 }
 #endif /* RECONSTRUCTPROC0MPI_H */
