@@ -40,9 +40,11 @@ PYBIND11_MODULE(StOptGeners, m)
     .def("readGridAndRegressedCuts", &BinaryFileArchiveStOpt::readGridAndRegressedCuts)
     .def("dumpGridTreeValue", &BinaryFileArchiveStOpt::dumpGridTreeValue)
     .def("readGridTreeValue", &BinaryFileArchiveStOpt::readGridTreeValue)
+    .def("dumpSome1DArray", &BinaryFileArchiveStOpt::dumpSome1DArray)
     .def("dumpSome2DArray", &BinaryFileArchiveStOpt::dumpSome2DArray)
     .def("dumpSomeRegressor", &BinaryFileArchiveStOpt::dumpSomeRegressor)
     .def("dumpSomeStringVector", &BinaryFileArchiveStOpt::dumpSomeStringVector)
+    .def("readSome1DArray", &BinaryFileArchiveStOpt::readSome1DArray)
     .def("readSome2DArray", &BinaryFileArchiveStOpt::readSome2DArray)
     .def("readSomeRegressor", &BinaryFileArchiveStOpt::readSomeRegressor)
     .def("readSomeStringVector", &BinaryFileArchiveStOpt::readSomeStringVector)
