@@ -77,17 +77,19 @@ double accuracyStopSDDP =  5.;
 /// \param p_nbPtGrid              number of grid points
 /// \param p_nbSimulcheck          number of simulations to check convergence
 /// \param p_initStorage           initial storage level
+/// \param p_world       MPI communicator
+/// \param p_strAdd      to make difference for file with  different communicators
 double valorizationDP(const size_t &p_nbStep,  const int &p_nbsimulOpt, const int   &p_nMeshDP,
                       const double &p_maxLevelStorage, const double &p_injectionRateStorage, const double &p_withdrawalRateStorage,
                       const double &p_injectionCostStorage, const   double &p_withdrawalCostStorage, const double &p_maturity,
                       shared_ptr<OneDimData<OneDimRegularSpaceGrid, double> > &p_futureGrid,
                       const double &p_mr, const double &p_sig, const int &p_nbPtGrid, const int &p_nbSimulcheck,
-                      const double   &p_initStorage)
-{
+                      const double   &p_initStorage
 #ifdef USE_MPI
-    boost::mpi::communicator world;
+                      , const boost::mpi::communicator &p_world, const  string &p_strAdd
 #endif
-
+                     )
+{
     // one dimensional factors
     int nDim = 1;
     VectorXd sigma = VectorXd::Constant(nDim, p_sig);
@@ -130,16 +132,19 @@ double valorizationDP(const size_t &p_nbStep,  const int &p_nbsimulOpt, const in
 
     // Optimize
     ///////////
-    string fileToDump = "CondExpGasStorage";
+    string fileToDump = "CondExpGasStorageSDDPTree";
+#ifdef USE_MPI
+    fileToDump = fileToDump + p_strAdd;
+#endif
     // link the simulations to the optimizer
     storage->setSimulator(backSimulatorDP);
 
 #ifdef USE_MPI
     bool bOneFile = true;
-    double valueOptim =  DynamicProgrammingByRegressionDist(grid, storage, regressor, vFunction, initialStockDP, initialRegime, fileToDump, bOneFile);
+    double valueOptim =  DynamicProgrammingByRegressionDist(grid, storage, regressor, vFunction, initialStockDP, initialRegime, fileToDump, bOneFile, p_world);
     // link the simulations to the optimizer
     storage->setSimulator(forSimulator);
-    double valSimu = SimulateRegressionDist(grid, storage, vFunction, initialStockDP, initialRegime, fileToDump, bOneFile) ;
+    double valSimu = SimulateRegressionDist(grid, storage, vFunction, initialStockDP, initialRegime, fileToDump, bOneFile, p_world) ;
 #else
     double valueOptim =  DynamicProgrammingByRegression(grid, storage, regressor, vFunction, initialStockDP, initialRegime, fileToDump);
     // link the simulations to the optimizer
@@ -149,7 +154,7 @@ double valorizationDP(const size_t &p_nbStep,  const int &p_nbsimulOpt, const in
 #endif
 
 #ifdef USE_MPI
-    if (world.rank() == 0)
+    if (p_world.rank() == 0)
 #endif
         cout << " Value by DP " << valueOptim << " AND  sim " <<  valSimu << endl ;
 
@@ -172,18 +177,24 @@ double valorizationDP(const size_t &p_nbStep,  const int &p_nbsimulOpt, const in
 /// \param p_nbSimul               number od sample in simulation
 /// \return value and optimization and simulation
 /// \param p_initStorage           initial storage level
+/// \param p_world       MPI communicator
+/// \param p_strAdd      to make difference for file with  different communicators
 double valorizationDPCuts(const int &p_nbStorage,
                           const double &p_maxLevelStorage, const double &p_injectionRateStorage, const double &p_withdrawalRateStorage,
                           const double &p_injectionCostStorage, const   double &p_withdrawalCostStorage,
                           shared_ptr<OneDimData<OneDimRegularSpaceGrid, double> > &p_futureGrid,
                           const string &p_name,
                           const double &p_mr, const double &p_sig, const int &p_nbPtGrid, const int &p_nbSimul,
-                          const double   &p_initStorage)
+                          const double   &p_initStorage
+#ifdef USE_MPI
+                          , const boost::mpi::communicator &p_world, const  string &p_strAdd
+#endif
+                         )
 {
 
 #ifdef USE_MPI
-    boost::mpi::communicator world;
-    int iTask = world.rank();
+    int iTask = p_world.rank();
+    p_world.barrier();
 #else
     int iTask = 0;
 #endif
@@ -217,8 +228,12 @@ double valorizationDPCuts(const int &p_nbStorage,
     ArrayXd dates = backSimulator->getDates();
 
     // names for archive
-    string nameCut = "CutGasStorage";
-    string nameVisitedStates = "VisitedStateGasStorage";
+    string nameCut = "CutGasStorageForTree";
+    string nameVisitedStates = "VisitedStateGasStorageForTree";
+#ifdef USE_MPI
+    nameCut = nameCut + p_strAdd;
+    nameVisitedStates = nameVisitedStates + p_strAdd;
+#endif
 
     // archive for cuts read write
     std::shared_ptr<gs::BinaryFileArchive> archiveForCuts;
@@ -261,31 +276,39 @@ double valorizationDPCuts(const int &p_nbStorage,
         }
     }
 #ifdef USE_MPI
-    world.barrier();
+    p_world.barrier();
 #endif
 
     // reinitialize simulator
     simBack->resetTime();
     ostringstream stringStream;
-    double  value = backwardSDDPTree(optimizer, simBack, dates, initialStock, finCut, nameVisitedStates, archiveForCuts);
+    double  value = backwardSDDPTree(optimizer, simBack, dates, initialStock, finCut, nameVisitedStates, archiveForCuts
+#ifdef USE_MPI
+                                     , p_world
+#endif
+                                    );
 
 #ifdef USE_MPI
-    if (world.rank() == 0)
+    if (p_world.rank() == 0)
 #endif
         cout << "End of backward with cuts" << endl ;
 #ifdef USE_MPI
-    world.barrier();
+    p_world.barrier();
 #endif
 
     // value in simulaton
     std::shared_ptr<SimulatorSDDPBaseTree>  simFor = static_pointer_cast<SimulatorSDDPBaseTree>(forSimulator);
 
-    double valSimul = forwardSDDPTree(optimizer, simFor, dates, initialStock, finCut, false, archiveForCuts, nameVisitedStates);
+    double valSimul = forwardSDDPTree(optimizer, simFor, dates, initialStock, finCut, false, archiveForCuts, nameVisitedStates
+#ifdef USE_MPI
+                                      , p_world
+#endif
+                                     );
 
 
 #ifdef USE_MPI
-    world.barrier();
-    if (world.rank() == 0)
+    p_world.barrier();
+    if (p_world.rank() == 0)
 #endif
     {
         cout << stringStream.str() << endl ;
@@ -293,7 +316,7 @@ double valorizationDPCuts(const int &p_nbStorage,
     }
 
     return valSimul;
-
+    return 0;
 }
 
 
@@ -312,6 +335,8 @@ double valorizationDPCuts(const int &p_nbStorage,
 /// \param p_mr                    mean reverting
 /// \param p_sig                   volatility
 /// \param p_initStorage           initial storage level
+/// \param p_world       MPI communicator
+/// \param p_strAdd      to make difference for file with  different communicators
 /// \return value and optimization and simulation
 pair<double, double> valorizationSDDP(const int &p_nbStorage, const size_t &p_nbStep, const int   &p_nbSamSimul, const int &p_nSimulCheck,
                                       const double &p_maxLevelStorage, const double &p_injectionRateStorage, const double &p_withdrawalRateStorage,
@@ -319,17 +344,19 @@ pair<double, double> valorizationSDDP(const int &p_nbStorage, const size_t &p_nb
                                       shared_ptr<OneDimData<OneDimRegularSpaceGrid, double> > &p_futureGrid,
                                       const string &p_name,
                                       const double &p_mr, const double &p_sig,
-                                      const double   &p_initStorage)
-{
+                                      const double   &p_initStorage
 #ifdef USE_MPI
-    boost::mpi::communicator world;
+                                      , const boost::mpi::communicator &p_world, const  string &p_strAdd
 #endif
+                                     )
+{
 
-
+#ifdef USE_MPI
+    p_world.barrier();
+#endif
     // initial values
     ArrayXd initialStock = ArrayXd::Constant(p_nbStorage, p_initStorage);
 
-
     shared_ptr<gs::BinaryFileArchive> binArxiv = make_shared<gs::BinaryFileArchive>(p_name.c_str(), "r");
 
     // create mean reverting backward simulator
@@ -354,8 +381,12 @@ pair<double, double> valorizationSDDP(const int &p_nbStorage, const size_t &p_nb
     ArrayXd dates = ArrayXd::LinSpaced(p_nbStep + 1, 0., p_maturity);
 
     // names for archive
-    string nameCut = "CutGasStorage";
-    string nameVisitedStates = "VisitedStateGasStorage";
+    string nameCut = "CutGasStorageTree";
+    string nameVisitedStates = "CutVisitedStateGasStorageTree";
+#ifdef USE_MPI
+    nameCut = nameCut + p_strAdd;
+    nameVisitedStates = nameVisitedStates + p_strAdd;
+#endif
 
     // precision parameter
     int nIterMax = 200;
@@ -363,9 +394,13 @@ pair<double, double> valorizationSDDP(const int &p_nbStorage, const size_t &p_nb
     int nstepIterations = 40; // take values between optimization n and optimization n+ nstepIterations for convergence criterion
     ostringstream stringStream;
     pair<double, double>  values = backwardForwardSDDPTree(optimizer, p_nSimulCheck, initialStock, finCut, dates,   nameCut, nameVisitedStates, nIterMax,
-                                   accuracy, nstepIterations, stringStream);
+                                   accuracy, nstepIterations, stringStream
 #ifdef USE_MPI
-    if (world.rank() == 0)
+                                   , p_world
+#endif
+                                                          );
+#ifdef USE_MPI
+    if (p_world.rank() == 0)
 #endif
     {
         cout << stringStream.str() << endl ;
@@ -385,14 +420,17 @@ pair<double, double> valorizationSDDP(const int &p_nbStorage, const size_t &p_nb
 /// \param p_nSimulCheck          number of simulations to check convergence in SDDP
 /// \param p_nbsimulOpt           number of simulations in backward resolution for DP resolution
 /// \param p_nMeshDP              number of mesh used in DP by regresson methods
+/// \param p_world                 MPI communicator
+/// \param p_strAdd                to make difference for file with  different communicators
 void testStorageSDDPTree(const int &p_nbStorage, const size_t &p_nbStep,  const int   &p_nbStepTreePerStep, const int   &p_nbSamSimul, const int &p_nSimulCheck,
                          const int &p_nbsimulOpt,
-                         const int   &p_nMeshDP)
+                         const int   &p_nMeshDP
+#ifdef USE_MPI
+                         , const boost::mpi::communicator &p_world, const  string &p_strAdd
+#endif
+                        )
 {
 
-#ifdef USE_MPI
-    boost::mpi::communicator world;
-#endif
 
     // storage
     /////////
@@ -428,11 +466,11 @@ void testStorageSDDPTree(const int &p_nbStorage, const size_t &p_nbStep,  const
     //**************
     double valueSimu = valorizationDP(p_nbStep,  p_nbsimulOpt, p_nMeshDP, maxLevelStorage,
                                       injectionRateStorage, withdrawalRateStorage, injectionCostStorage, withdrawalCostStorage,
-                                      maturity, futureGrid, mr, sigma, nbPtGrid, p_nSimulCheck, initLevel);
+                                      maturity, futureGrid, mr, sigma, nbPtGrid, p_nSimulCheck, initLevel, p_world, p_strAdd);
 
 
 #ifdef USE_MPI
-    world.barrier();
+    p_world.barrier();
 #endif
 
 
@@ -451,22 +489,22 @@ void testStorageSDDPTree(const int &p_nbStorage, const size_t &p_nbStep,  const
     // create archive
     string nameTree = "Tree";
 #ifdef USE_MPI
-    if (world.rank() == 0)
+    if (p_world.rank() == 0)
 #endif
         tree.dump(nameTree, indexT);
 #ifdef USE_MPI
-    world.barrier();
+    p_world.barrier();
 #endif
 
 
     // solve SDDP
     pair<double, double> values = valorizationSDDP(p_nbStorage, p_nbStep, p_nbSamSimul, p_nSimulCheck, maxLevelStorage, injectionRateStorage, withdrawalRateStorage,
-                                  injectionCostStorage, withdrawalCostStorage, maturity, futureGrid, nameTree, mr, sigma, initLevel);
+                                  injectionCostStorage, withdrawalCostStorage, maturity, futureGrid, nameTree, mr, sigma, initLevel, p_world, p_strAdd);
 
 
 #ifdef USE_MPI
-    world.barrier();
-    if (world.rank() == 0)
+    p_world.barrier();
+    if (p_world.rank() == 0)
 #endif
     {
         BOOST_CHECK_CLOSE(valueSimu * p_nbStorage, values.second, accuracyCClose);
@@ -478,12 +516,14 @@ void testStorageSDDPTree(const int &p_nbStorage, const size_t &p_nbStep,  const
 ///         Here deterministic tree (one node)
 /// \param p_nbStep               number of time step for storage valuation
 /// \param p_nbPtGrid             number of grid points
-void testStorageDPDeterTreeCut(const size_t &p_nbStep, const int &p_nbPtGrid)
-{
-
+/// \param p_world       MPI communicator
+/// \param p_strAdd      to make difference for file with  different communicators
+void testStorageDPDeterTreeCut(const size_t &p_nbStep, const int &p_nbPtGrid
 #ifdef USE_MPI
-    boost::mpi::communicator world;
+                               , const boost::mpi::communicator &p_world, const  string &p_strAdd
 #endif
+                              )
+{
 
     // storage
     /////////
@@ -520,11 +560,11 @@ void testStorageDPDeterTreeCut(const size_t &p_nbStep, const int &p_nbPtGrid)
     int nbmesh = 1;
     double valueSimu = valorizationDP(p_nbStep, nbsim, nbmesh, maxLevelStorage,
                                       injectionRateStorage, withdrawalRateStorage, injectionCostStorage, withdrawalCostStorage,
-                                      maturity, futureGrid, mr, sig, p_nbPtGrid, 1, initLevel);
+                                      maturity, futureGrid, mr, sig, p_nbPtGrid, 1, initLevel, p_world, p_strAdd);
 
 
 #ifdef USE_MPI
-    world.barrier();
+    p_world.barrier();
 #endif
 
 
@@ -541,29 +581,31 @@ void testStorageDPDeterTreeCut(const size_t &p_nbStep, const int &p_nbPtGrid)
 
     // create arxiv
     string nameTree = "Tree";
+
 #ifdef USE_MPI
-    if (world.rank() == 0)
+    nameTree = nameTree + p_strAdd;
+    if (p_world.rank() == 0)
 #endif
         tree.dump(nameTree, indexT);
 
 #ifdef USE_MPI
-    world.barrier();
+    p_world.barrier();
 #endif
 
     // solve DP with cuts
     double valDPCut = valorizationDPCuts(1, maxLevelStorage, injectionRateStorage, withdrawalRateStorage,
-                                         injectionCostStorage,  withdrawalCostStorage,  futureGrid, nameTree, mr, sig, p_nbPtGrid, 1, initLevel);
+                                         injectionCostStorage,  withdrawalCostStorage,  futureGrid, nameTree, mr, sig, p_nbPtGrid, 1, initLevel, p_world, p_strAdd);
 
 #ifdef USE_MPI
-    world.barrier();
-    if (world.rank() == 0)
+    p_world.barrier();
+    if (p_world.rank() == 0)
 #endif
     {
         BOOST_CHECK_CLOSE(valueSimu, valDPCut, accuracyClose);
     }
 
 #ifdef USE_MPI
-    world.barrier();
+    p_world.barrier();
 #endif
 
 
@@ -576,14 +618,18 @@ void testStorageDPDeterTreeCut(const size_t &p_nbStep, const int &p_nbPtGrid)
 /// \param p_nbsimulOpt           number of simulations in backward resolution for DP resolution
 /// \param p_nMeshDP              number of mesh used in DP by regresson methods
 /// \param p_nbPtGrid             number of points on grid
+/// \param p_world       MPI communicator
+/// \param p_strAdd      to make difference for file with  different communicators
 void testStorageCutTree(const size_t &p_nbStep,  const int   &p_nbStepTreePerStep,  const int &p_nSimulCheck,
                         const int &p_nbsimulOpt,
-                        const int   &p_nMeshDP, int p_nbPtGrid)
+                        const int   &p_nMeshDP, int p_nbPtGrid
+#ifdef USE_MPI
+                        , const boost::mpi::communicator &p_world, const  string &p_strAdd
+#endif
+
+                       )
 {
 
-#ifdef USE_MPI
-    boost::mpi::communicator world;
-#endif
 
     // storage
     /////////
@@ -618,11 +664,11 @@ void testStorageCutTree(const size_t &p_nbStep,  const int   &p_nbStepTreePerSte
     //**************
     double valueSim = valorizationDP(p_nbStep,  p_nbsimulOpt, p_nMeshDP, maxLevelStorage,
                                      injectionRateStorage, withdrawalRateStorage, injectionCostStorage, withdrawalCostStorage,
-                                     maturity, futureGrid, mr, sigma, p_nbPtGrid, p_nSimulCheck, initLevel);
+                                     maturity, futureGrid, mr, sigma, p_nbPtGrid, p_nSimulCheck, initLevel, p_world, p_strAdd);
 
 
 #ifdef USE_MPI
-    world.barrier();
+    p_world.barrier();
 #endif
 
 
@@ -641,21 +687,22 @@ void testStorageCutTree(const size_t &p_nbStep,  const int   &p_nbStepTreePerSte
     // create arxiv
     string nameTree = "Tree";
 #ifdef USE_MPI
-    if (world.rank() == 0)
+    nameTree = nameTree + p_strAdd;
+    if (p_world.rank() == 0)
 #endif
         tree.dump(nameTree, indexT);
 
 #ifdef USE_MPI
-    world.barrier();
+    p_world.barrier();
 #endif
 
     // solve DP with cuts
     double valDPCut = valorizationDPCuts(1, maxLevelStorage, injectionRateStorage, withdrawalRateStorage,
                                          injectionCostStorage,  withdrawalCostStorage,  futureGrid, nameTree,
-                                         mr, sigma, p_nbPtGrid, p_nSimulCheck, initLevel);
+                                         mr, sigma, p_nbPtGrid, p_nSimulCheck, initLevel, p_world, p_strAdd);
 
 #ifdef USE_MPI
-    if (world.rank() == 0)
+    if (p_world.rank() == 0)
 #endif
     {
         BOOST_CHECK_CLOSE(valueSim, valDPCut, accuracyCClose);
@@ -669,43 +716,76 @@ BOOST_AUTO_TEST_CASE(testSimpleStorageDeterministicCutTree)
 {
     int nbStep = 30 ;
     int nbPtGrid = 200;
-    testStorageDPDeterTreeCut(nbStep, nbPtGrid);
+#ifdef USE_MPI
+    boost::mpi::communicator world ;
+    world.barrier();
+#endif
+    testStorageDPDeterTreeCut(nbStep, nbPtGrid
+#ifdef USE_MPI
+                              , world, ""
+#endif
+                             );
 }
 
+#ifdef USE_MPI
+BOOST_AUTO_TEST_CASE(testSimpleStorageDeterministicCutTreeCommunicator)
+{
+    int nbStep = 30 ;
+    int nbPtGrid = 200;
+    boost::mpi::communicator world ;
+    world.barrier();
+    if (world.size() > 1)
+    {
+        // create 2 communicators
+        bool bSeparate = (2 * world.rank() < world.size());
+        boost::mpi::communicator worldLoc = world.split(bSeparate ? 0 : 1);
+        string strToAdd = (bSeparate ? "0" : "1");
+        testStorageDPDeterTreeCut(nbStep, nbPtGrid, worldLoc, strToAdd);
+    }
+    world.barrier();
+}
+#endif
+
+
 BOOST_AUTO_TEST_CASE(testSimpleStorageCutTree)
 {
+#ifdef USE_MPI
+    boost::mpi::communicator world ;
+    world.barrier();
+#endif
     int nbsimulLinear = 10000;
     int nbStep = 30 ;
     int nbStepTreePerStep = 1;
     int nbMeshLinear = 6 ;
     int nSimulCheck = 10000;
     int nbPtGrid = 200;
-    testStorageCutTree(nbStep, nbStepTreePerStep,   nSimulCheck,  nbsimulLinear, nbMeshLinear, nbPtGrid);
+    testStorageCutTree(nbStep, nbStepTreePerStep,   nSimulCheck,  nbsimulLinear, nbMeshLinear, nbPtGrid
+#ifdef USE_MPI
+                       , world, ""
+#endif
+                      );
 }
 
 
 BOOST_AUTO_TEST_CASE(testSimpleStorageSDDPTree1D1Step)
 {
+#ifdef USE_MPI
+    boost::mpi::communicator world ;
+    world.barrier();
+#endif
     int nbsimulLinear = 20000;
     int nbSamSimulLinear = 5;
     int nbStep = 30 ;
     int nbStepTreePerStep = 1;
     int nbMeshLinear = 6 ;
     int nSimulCheck = 10000;
-    testStorageSDDPTree(1, nbStep, nbStepTreePerStep,  nbSamSimulLinear, nSimulCheck,  nbsimulLinear, nbMeshLinear);
+    testStorageSDDPTree(1, nbStep, nbStepTreePerStep,  nbSamSimulLinear, nSimulCheck,  nbsimulLinear, nbMeshLinear
+#ifdef USE_MPI
+                        , world, ""
+#endif
+                       );
 }
 
-// BOOST_AUTO_TEST_CASE(testSimpleStorageSDDPTree1D2Step)
-// {
-//     int nbsimulLinear = 20000;
-//     int nbSamSimulLinear = 10;
-//     int nbStep = 30 ;
-//     int nbStepTreePerStep = 2;
-//     int nbMeshLinear = 6 ;
-//     int nSimulCheck = 10000;
-//     testStorageSDDPTree(1, nbStep, nbStepTreePerStep,  nbSamSimulLinear, nSimulCheck,  nbsimulLinear, nbMeshLinear);
-
-// }
 
 
 #ifdef USE_MPI
