@@ -6,12 +6,15 @@
 #include "StOpt/core/utils/constant.h"
 
 using namespace std;
+using namespace Eigen;
+
+double accuracyEqual = 1e-10;
 
 BOOST_AUTO_TEST_CASE(StOptRegularSpaceGrid)
 {
     size_t nsteps = 1000;
     double step = 400.0 / (1.0 * nsteps);
-    Eigen::ArrayXd minimum(1);
+    ArrayXd minimum(1);
     for (size_t iPoint = 0; iPoint <= nsteps; iPoint++)
     {
         minimum(0) = iPoint * step;
@@ -20,13 +23,13 @@ BOOST_AUTO_TEST_CASE(StOptRegularSpaceGrid)
     }
     //minimum(0) \approx 135.20000000000002;
 
-    Eigen::ArrayXd step_grid(1);
+    ArrayXd step_grid(1);
     step_grid(0) = (140. - minimum(0));
-    Eigen::ArrayXi nsteps_grid(1);
+    ArrayXi nsteps_grid(1);
     nsteps_grid(0) = 1;
     StOpt::RegularSpaceGrid grid = StOpt::RegularSpaceGrid(minimum, step_grid, nsteps_grid);
 
-    Eigen::ArrayXd point(1);
+    ArrayXd point(1);
     point(0) = 135.19999999999999;
 
     std::stringstream ss_minimum;
@@ -38,7 +41,21 @@ BOOST_AUTO_TEST_CASE(StOptRegularSpaceGrid)
     cout  << "grid.isInside(point): " << grid.isInside(point);
     if (grid.isInside(point))
     {
-        Eigen::ArrayXi result = grid.lowerPositionCoord(point); // Assertion fails.
+        ArrayXi result = grid.lowerPositionCoord(point); // Assertion fails.
         cout << "grid.lowerPositionCoord(point): " << result;
     }
+
+     // get back 1D grids in each dimension
+    vector<shared_ptr<ArrayXd>> vec1DGrids = grid.get1DGrids();
+    for (size_t id =0; id < vec1DGrids.size(); ++id)
+      cout << "1D grids " <<    *vec1DGrids[id] << endl ;
+    shared_ptr<StOpt::GridIterator> iterGenGrid = grid.getGridIterator();
+    int ip =0; 
+    while (iterGenGrid->isValid())
+    {
+      ArrayXd point = iterGenGrid->getCoordinate();
+      BOOST_CHECK_CLOSE(point(0), (*vec1DGrids[0])(ip), accuracyEqual);
+      ip +=1;
+      iterGenGrid->next();
+   }
 }
