@@ -403,6 +403,34 @@ void TestContinuationValue(const int &p_nDim, const int &p_nbSimul, const int &p
         BinaryFileArchive ar("archiveLLR1", "w");
         ar << Record(continuation, "FirstContinuation", "Top") ;
 
+	// iterate on grid points
+	// back grid iterator from continuation value object
+	shared_ptr<GridIterator> iterRegGrid  =continuation.getGrid()->getGridIterator();
+	// get back the condition expectation operator
+	std::shared_ptr<BaseRegression > condExp = continuation.getCondExp();
+	// get back basis functions for all grid points
+	const Eigen::ArrayXXd & basisFuncValues= continuation.getValues();
+	// stock identifier
+	int iStock =0;
+	while (iterRegGrid->isValid())
+	  {
+	    // first version using the stock level  so with interpolation
+	     ArrayXd pointCoord = iterRegGrid->getCoordinate();
+	     ArrayXd regressedByContinuation1 = continuation.getAllSimulations(pointCoord);
+	     // second method without interpolation : get back directly the basis function associated to the stock identifier
+	     ArrayXd regressedByContinuation2 =condExp->reconstruction(basisFuncValues.col(iStock));
+	     // third method to be used in simulation
+	     ArrayXd regressedByContinuation3(p_nbSimul);
+	     for (int is  = 0;  is < p_nbSimul; ++is)
+	       regressedByContinuation3(is)=  condExp->getValue(x.col(is),basisFuncValues.col(iStock));
+	     for (int is  = 0;  is < p_nbSimul; ++is)
+	       {
+		 BOOST_CHECK_CLOSE(regressedByContinuation1(is), regressedByContinuation2(is), accuracyEqual);
+		 BOOST_CHECK_CLOSE(regressedByContinuation1(is), regressedByContinuation3(is), accuracyEqual);
+	       }
+	     iterRegGrid->next();
+	     iStock += 1;
+	  }
     }
     {
         // read archive
